@@ -1,7 +1,14 @@
 import logging
 import threading
 
-_format_string = "%(asctime)s %(levelname)s %(logLabel)s %(filename)s:%(lineno)d %(message)s"
+SPARK_AVAILABLE = False
+try:
+  from pyspark import TaskContext
+  SPARK_AVAILABLE = True
+except:
+  pass
+
+_format_string = "%(asctime)s %(logLabel)s %(levelname)s %(filename)s:%(lineno)d %(message)s"
 
 _thread_local = threading.local()
 
@@ -32,12 +39,18 @@ def get_log_label():
   try:
     return _thread_local.log_label
   except AttributeError:
+    if SPARK_AVAILABLE:
+      ctx = TaskContext.get()
+      if ctx:
+        return ctx.getLocalProperty('mdc.logLabel')
     return ''
 
 
 class LabelFilter(logging.Filter):
   def filter(self, record):
     record.logLabel = get_log_label()
+    if record.levelno == logging.WARNING:
+      record.levelname = 'WARN'
     return True
 
 
