@@ -1,18 +1,34 @@
 """Ascend resource applier classes - used for applying resource definitions."""
 
+import sys
+import uuid
+from typing import Dict, List
+
+import backoff
 import glog
 import networkx as nx
-from typing import List, Dict
-import uuid
 
 import ascend.protos.api.api_pb2 as api_pb2
 from ascend.sdk.client import Client
-from ascend.sdk.common import components_ordered_by_dependency, dataflows_ordered_by_dependency
-from ascend.sdk.definitions import (Component, Connection, Credential, Dataflow, ReadConnector, Transform, WriteConnector, ComponentGroup, ComponentIdToUuidMap,
-                                    ComponentUuidType, DataFeed, DataFeedConnector, DataService)
+from ascend.sdk.common import (components_ordered_by_dependency, dataflows_ordered_by_dependency)
+from ascend.sdk.definitions import (Component, ComponentGroup, ComponentIdToUuidMap, ComponentUuidType, Connection, Credential, DataFeed, DataFeedConnector,
+                                    Dataflow, DataService, ReadConnector, Transform, WriteConnector)
 from openapi_client.exceptions import ApiException
 
 
+def on_backoff(backoff_details):
+  exc_type, exc_value, _ = sys.exc_info()
+  glog.warn(f"retrying after {backoff_details.get('tries')} attempt due to: {exc_type} {exc_value}")
+
+
+def fatal_code(e: ApiException):
+  return e.status not in [502, 504]
+
+
+class RetriableListError(Exception):
+  pass
+
+
 class DataServiceApplier:
   """DataServiceApplier is a utility class that accepts a DataService definition and
   'applies' it - ensuring that the DataService is created if it does not already exist,
@@ -24,6 +40,7 @@ class DataServiceApplier:
     """Creates a new DataServiceApplier."""
     self.client = client
 
+  @backoff.on_exception(exception=RetriableListError, wait_gen=backoff.expo, max_value=5, max_time=30, jitter=None, on_backoff=on_backoff)
   def apply(self, data_service: DataService, delete=True, dry_run=False, after_credentials=None, after_connections=None):
     """
     Create or update the specified DataService.
@@ -41,19 +58,22 @@ class DataServiceApplier:
     try:
       self.client.get_data_service(data_service.id)
       exists = True
+
+      if exists:
+        glog.info(f"Update DataService: {data_service.id}")
+        if not dry_run:
+          _tracked_call(self.client.update_data_service, data_service.id, data_service.to_proto())
+      else:
+        if not dry_run:
+          glog.info(f"Create DataService: {data_service.id}")
+          _tracked_call(self.client.create_data_service, data_service.to_proto())
+
     except ApiException as e:
+      if not fatal_code(e):
+        raise RetriableListError(str(e))
       if e.status != 404:
         raise e
 
-    if exists:
-      glog.info(f"Update DataService: {data_service.id}")
-      if not dry_run:
-        _tracked_call(self.client.update_data_service, data_service.id, data_service.to_proto())
-    else:
-      if not dry_run:
-        glog.info(f"Create DataService: {data_service.id}")
-        _tracked_call(self.client.create_data_service, data_service.to_proto())
-
     if data_service.credentials is not None:
       for i, cred in enumerate(data_service.credentials):
         data_service.credentials[i] = CredentialApplier(self.client).apply(data_service.id, cred, dry_run)
@@ -121,6 +141,7 @@ class CredentialApplier:
   def __init__(self, client: Client):
     self.client = client
 
+  @backoff.on_exception(exception=ApiException, wait_gen=backoff.expo, max_value=5, max_time=30, jitter=None, on_backoff=on_backoff, giveup=fatal_code)
   def apply(self, data_service_id, credential: Credential, dry_run=False):
     glog.info(f"Apply Credential: (ds={data_service_id} credential_id={credential.id} name={credential.name})")
     existing = None
@@ -153,6 +174,7 @@ class ConnectionApplier:
   def __init__(self, client: Client):
     self.client = client
 
+  @backoff.on_exception(exception=ApiException, wait_gen=backoff.expo, max_value=5, max_time=30, jitter=None, on_backoff=on_backoff, giveup=fatal_code)
   def apply(self, data_service_id, connection: Connection, dry_run=False):
     glog.info(f"Apply Connection: (ds={data_service_id} id={connection.id} name={connection.name})")
     existing = None
@@ -331,6 +353,7 @@ class GroupApplier:
     self.client = client
     self.id_map = id_map
 
+  @backoff.on_exception(exception=ApiException, wait_gen=backoff.expo, max_value=5, max_time=30, jitter=None, on_backoff=on_backoff, giveup=fatal_code)
   def apply(self, data_service_id, dataflow_id, group: ComponentGroup, dry_run=False):
     glog.info(f"Apply ComponentGroup: (ds={data_service_id} df={dataflow_id} {group.id})")
     exists = False
@@ -365,6 +388,7 @@ class DataFeedApplier:
     self.client = client
     self.id_map = id_map
 
+  @backoff.on_exception(exception=ApiException, wait_gen=backoff.expo, max_value=5, max_time=30, jitter=None, on_backoff=on_backoff, giveup=fatal_code)
   def apply(self, data_service_id, dataflow_id, data_feed: DataFeed, dry_run=False):
     glog.info(f"Apply DataFeed: (ds={data_service_id} df={dataflow_id} {data_feed.id})")
     exists = False
@@ -402,6 +426,7 @@ class DataFeedConnectorApplier:
   def __init__(self, client: Client):
     self.client = client
 
+  @backoff.on_exception(exception=ApiException, wait_gen=backoff.expo, max_value=5, max_time=30, jitter=None, on_backoff=on_backoff, giveup=fatal_code)
   def apply(self, data_service_id, dataflow_id, data_feed_connector: DataFeedConnector, dry_run=False):
     try:
       data_feed = self.client.get_data_feed(data_feed_connector.input_data_service_id, data_feed_connector.input_dataflow_id,
@@ -524,6 +549,7 @@ class ComponentApplier:
     else:
       return default
 
+  @backoff.on_exception(exception=ApiException, wait_gen=backoff.expo, max_value=5, max_time=30, jitter=None, on_backoff=on_backoff, giveup=fatal_code)
   def apply(self, data_service_id, dataflow_id, component: Component):
     """
     Applies the provided component and
