@@ -11,6 +11,7 @@ import ascend.protos.ascend.ascend_pb2 as ascend_pb2
 import ascend.protos.component.component_pb2 as component_pb2
 import ascend.protos.connection.connection_pb2 as connection_pb2
 import ascend.protos.core.core_pb2 as core_pb2
+import ascend.protos.expression.expression_pb2 as expression_pb2
 import ascend.protos.io.io_pb2 as io_pb2
 import ascend.protos.operator.operator_pb2 as operator_pb2
 import ascend.protos.pattern.pattern_pb2 as pattern_pb2
@@ -56,7 +57,9 @@ class ReadConnector(Component):
                last_manual_refresh_time: timestamp_pb2.Timestamp = None,
                aggregation_limit: wrappers_pb2.Int64Value = None,
                bytes: component_pb2.Source.FromBytes = None,
-               records: component_pb2.Source.FromRecords = None):
+               records: component_pb2.Source.FromRecords = None,
+               compute_configurations: List[expression_pb2.StageComputeConfiguration] = None,
+               non_materialized: bool = None):
     """
     Create a ReadConnector definition.
 
@@ -73,6 +76,9 @@ class ReadConnector(Component):
     - bytes: defines the operator for parsing bytes read from the source (example - json
       or xsv parsing)
     - records: defines the schema for records read from a database or warehouse
+    - compute_configurations: custom configuration overrides by stage. Currently supports the
+      Read Stage for connections-type ReadConnectors
+    - non_materialized: if set to true, the contents of the source are not materialized
     Returns: a new ReadConnector instance.
 
     Note - a read connector can have either `bytes` or `records` defined, but not both.
@@ -88,6 +94,8 @@ class ReadConnector(Component):
     self.aggregation_limit = aggregation_limit
     self.bytes = bytes
     self.records = records
+    self.compute_configurations = compute_configurations
+    self.non_materialized = non_materialized
 
   def dependencies(self) -> List[str]:
     return []
@@ -108,6 +116,10 @@ class ReadConnector(Component):
       proto.source.bytes.CopyFrom(self.bytes)
     if self.records is not None:
       proto.source.records.CopyFrom(self.records)
+    if self.compute_configurations is not None:
+      proto.source.compute_configurations.extend(self.compute_configurations)
+    if self.non_materialized is not None:
+      proto.source.non_materialized = self.non_materialized
 
     return proto
 
@@ -131,6 +143,9 @@ class ReadConnector(Component):
       rc.assigned_priority = proto.source.assigned_priority
     if proto.source.HasField("aggregation_limit"):
       rc.aggregation_limit = proto.source.aggregation_limit
+    if len(proto.source.compute_configurations) > 0:
+      rc.compute_configurations = proto.source.compute_configurations[:]
+    rc.non_materialized = proto.source.non_materialized
     return rc
 
 
@@ -209,7 +224,8 @@ class WriteConnector(Component):
                description: str = None,
                assigned_priority: component_pb2.Priority = None,
                bytes: component_pb2.Sink.ToBytes = None,
-               records: component_pb2.Sink.ToRecords = None):
+               records: component_pb2.Sink.ToRecords = None,
+               compute_configurations: List[expression_pb2.StageComputeConfiguration] = None):
     """
     Create a WriteConnector definition.
 
@@ -222,7 +238,9 @@ class WriteConnector(Component):
     - assigned_priority: scheduling priority of write connector
     - bytes: defines the operator for formatting bytes written to the object store destination
     - records: indicates that the destination is a structured record store like a database
-    or a data warehouse
+      or a data warehouse
+    - compute_configurations: custom configuration overrides by stage. Currently supports the
+      Write Stage for connections-type WriteConnectors
     Returns: a new WriteConnector instance.
 
     Note - a write connector can have either `bytes` or `records` defined, but not both.
@@ -235,6 +253,7 @@ class WriteConnector(Component):
     self.assigned_priority = assigned_priority
     self.bytes = bytes
     self.records = records
+    self.compute_configurations = compute_configurations
 
   def dependencies(self) -> List[str]:
     return [self.input_id]
@@ -253,6 +272,8 @@ class WriteConnector(Component):
       proto.sink.bytes.CopyFrom(self.bytes)
     if self.records is not None:
       proto.sink.records.CopyFrom(self.records)
+    if self.compute_configurations is not None:
+      proto.sink.compute_configurations.extend(self.compute_configurations)
 
     return proto
 
@@ -269,6 +290,8 @@ class WriteConnector(Component):
         wc.records = proto.sink.records
       if proto.sink.HasField("assigned_priority"):
         wc.assigned_priority = proto.sink.assigned_priority
+      if len(proto.sink.compute_configurations) > 0:
+        wc.compute_configurations = proto.sink.compute_configurations[:]
       return wc
     except KeyError as e:
       raise KeyError(f"unresolved id for component {e}")
