@@ -41,8 +41,14 @@ def Timestamp(value=None) -> ascend.Value:
   return ascend.Value(timestamp_value=value)
 
 
-def Union(tag, value=Struct(), fields=None) -> ascend.Value:
-  if fields is not None:
-    return ascend.Value(union_value=ascend.Union(tag=tag, value=Struct(fields)))
+def Union(tag=None, fields=None, **kwargs):
+  if tag is not None:
+    assert not kwargs, f"Union should be defined in one of two ways: Union(tag='my_option', fields=my_option_fields) or Union(my_option=my_option_value)"
+    union_value = ascend.Union(tag=tag, value=fields if isinstance(fields, ascend.Value) else Struct(fields or {}))
   else:
-    return ascend.Value(union_value=ascend.Union(tag=tag, value=value))
+    assert fields is None and len(kwargs) <= 1, \
+      f"Union should be defined in one of two ways: Union(tag='my_option', fields=my_option_fields) or Union(my_option=my_option_value)"
+    maybe_case = next(iter(kwargs.items()), None)
+    union_value = ascend.Union() if maybe_case is None else ascend.Union(tag=maybe_case[0], value=Struct(maybe_case[1]))
+
+  return ascend.Value(union_value=union_value)
